import { Ref, ref } from 'vue'
import { callApi, QueryParams } from '@/composables/engine/callApi'
import { start, stop } from '@/composables/engine/startStop'
import { fetchUpdate } from '@/composables/engine/fetchUpdate'

interface EngineContext {
    isActive: boolean
    shouldStop: boolean
    version: number
    lastFetched: Array<Date>
    isConnected: Ref<boolean>
    retry: Ref<number>
    fetchUpdate: () => void
    callApi: (url: string, queryParams?: QueryParams) => Promise<unknown>
}

export interface useEngine {
    isConnected: Ref<boolean>
    retry: Ref<number>
    start(): void
    stop(): void
    fetchUpdate(): void
}

export default (): useEngine => {
    const now = new Date(0)
    const context: EngineContext = {
        lastFetched: [now, now, now, now, now],
        isActive: false,
        shouldStop: false,
        version: -1,
        isConnected: ref(false),
        retry: ref(0),
        callApi,
        fetchUpdate,
    }

    return {
        isConnected: context.isConnected,
        retry: context.retry,
        start: () => start.apply(context),
        stop: () => stop.apply(context),
        fetchUpdate: () => fetchUpdate.apply(context),
    }
}